<?php
/**
 * $Horde: vacation/lib/Driver/sql.php,v 1.2.2.3 2003/01/20 06:03:04 ericr Exp $
 *
 * Copyright 2001-2003 Ilya Krel and Mike Cochrane
 *
 * See the enclosed file LICENSE for license information (BSD). If you
 * did not receive this file, see http://www.horde.org/bsdl.php.
 *
 * Vacation_Driver_ldap:: implements the Forwards_Driver API for LDAP servers.
 *
 * @author  Ilya Krel <mail@krel.org.org>
 * @author  Mike Cochrane <mike@graftonhall.co.nz>
 * @version $Revision: 1.2.2.3 $
 * @since   Vacation 2.1
 * @package vacation
 */

class Vacation_Driver_ldap extends Vacation_Driver {
  
    /** file pointer to the ldap connection. */
    var $ds;

    /** error string returned to user if an eror occurs. */
    var $err_str;

    /** boolean which contains state of ldap connection */
    var $connected=false;

    var $params;

    /**
     * Constructs a new ldap Vacation_Driver object.
     *
     * @param array  $params    A hash containing connection parameters.
     */

    function Vacation_Driver_ldap($params = array())
    {
        $this->params = $params;
    }


    /**
     * Do an ldap connect and login as user with privilege to change passwd.
     *
     * @return   boolean   True or False based on success of connect
     *
     */

    function _connect($user, $realm, $password)
    {

        if (!$this->connected) {
            if (!is_array($this->params)) {
                Horde::fatal(new PEAR_Error(
                _("No configuration information specified for LDAP authentication.")),
                __FILE__, __LINE__);
            }
            if (!isset($this->params['hostspec'])) {
                Horde::fatal(new PEAR_Error(
                _("Required 'hostspec' not specified in authentication configuration.")),
                __FILE__, __LINE__);
            }
            if (!isset($this->params['port'])) {
                Horde::fatal(new PEAR_Error(
                _("Required 'port' not specified in authentication configuration.")),
                __FILE__, __LINE__);
            }
	    /* Optional in ldap, autogenerated from search
            if (!isset($this->params['username'])) {
                Horde::fatal(new PEAR_Error(
                _("Required 'username' not specified in authentication configuration.")),
                __FILE__, __LINE__);
            }
            if (!isset($this->params['password'])) {
                Horde::fatal(new PEAR_Error(
                _("Required 'password' not specified in authentication configuration.")),
                __FILE__, __LINE__);
            }
	    */
            if (!isset($this->params['basedn'])) {
                Horde::fatal(new PEAR_Error(
                _("Required 'basedn' not specified in authentication configuration.")),
                __FILE__, __LINE__);
            }
	    if (!isset($this->params['search'])) {
	        Horde::fatal(new PEAR_Error(
		_("Required 'search' not specified in search configuration.")),
		__FILE__, __LINE__);
	    }

	    $this->ds = ldap_connect($this->params['hostspec'], $this->params['port']);
	    if (!$this->ds) {
	      return PEAR::raiseError(_("Could not connect to ldap server"));
	    }

	    if (isset($this->params['userdn']) &&
		isset($this->params['password'])) {
	        $result = @ldap_bind($this->ds, $this->params['userdn'], $this->params['password']);
	    } else {
	        $result = @ldap_bind($this->ds);
		// construct search
		$search = ereg_replace("%user%", $user, $this->params['search']);
		$search = ereg_replace("%realm%", $realm, $search);
	      
		// get userdn
		$result = ldap_search($this->ds, $this->params['basedn'], $search);
		$entry = ldap_first_entry($this->ds, $result);
		if ($entry === false) {
		    $this->_disconnect();
		    return PEAR::raiseError(_("User search gave nothing."));
		}
		$this->params['userdn'] = ldap_get_dn($this->ds, $entry);
		$this->params['password'] = $password;
	        $result = @ldap_bind($this->ds, $this->params['userdn'], $this->params['password']);
	    }
	    if (! $result) {
	        return PEAR::raiseError(_("Could not bind to ldap server"));
	    }
            $this->connected = true;
        }

        return true;
    }

  /**
     * Disconnect from the LDAP server and clean up the connection.
     *
     * @return boolean true on success, false on failure.
     */

    function _disconnect()
    {

        if ($this->connected) {
            $this->connected = false;
	    @ldap_close($this->ds);
        }

        return true;
    }


    /**
     * Get the vacation message
     *
     * @return string with vacation message
     */
    function _get_vacation() {
	$vac = $this->params['vacation'];
	$sr = ldap_search($this->ds, $this->params['userdn'], "$vac=*");
	$entry = ldap_first_entry($this->ds, $sr);
	if (!$entry) {
	  return false;
        }
	$values = ldap_get_values($this->ds, $entry, $vac);
	if ($values["count"] == 0) {
	   return false;
        }
	return $values[0];
    }

    /**
     * Begins forwarding of mail for a user.
     *
     * @param string    $user      The username to enable forwarding for.
     * @param string    $realm     The realm of the user.
     * @param string    $pass      The password of the user.
     * @param string    $target    The email address that mail should be
     *                             forwarded to.
     * @param optional boolean $keeplocal   A flag that if true causes a
     *                             copy of forwarded email to be kept in
     *                             the local mailbox.
     *
     * @return boolean  Returns true on success, false on error.
     */

    function set_vacation($user, $realm, $password, $message)
    {

        /* _connect() will die with Horde::fatal() upon failure. */
        $this->_connect($user, $realm, $password);

	/*
        // split message into parts to get subject
        $myarray = explode("\n",$message);

        // determine if $message  contains Subject: and if it does split it
        if ( strstr($myarray[0], "Subject:")) {
            $mysubject = $myarray[0];
            $myarray[0] = "" ;
            $mymessage = implode("", $myarray);
        } else {
            $mysubject = "" ;
            $mymessage = $message ;
        }
	*/

	// Prepare the message. \n->\n\n and UTF-8 encode.
	$message = ereg_replace("\r\n", "\\n", $message); 
	$message = mb_convert_encoding($message, "UTF-8", "ISO-8859-1");

	// change the user's vacation.
        $newDetails[$this->params['vacation']] = $message;
        $res = ldap_mod_replace($this->ds, $this->params['userdn'], $newDetails);
        $value = $this->_get_vacation();
        if (!$res) {
	    $this->err_str = ldap_error($this->ds);
            $res = PEAR::raiseError(ldap_error($this->ds));
	    $this->_disconnect;
	    return false;
        }

        $this->_disconnect();
        return true;

    }

    /**
     * Stops forwarding of mail for a user.
     *
     * @param string    $user      The username of the user.
     * @param string    $realm     The realm of the user.
     * @param string    $pass      The password of the user.
     *
     * @return boolean  Returns true on success, false on error.
     */

    function unset_vacation($user, $realm, $password)
    {
        // set forwarding flag
        /* _connect() will die with Horde::fatal() upon failure. */
        $this->_connect($user, $realm, $password);

	// del the user's vacation.
	$value = $this->_get_vacation();
	$newDetails[$this->params['vacation']] = $value;
        $res = ldap_mod_del($this->ds, $this->params['userdn'], $newDetails);
        if (!$res) {
	    $this->err_str = ldap_error($this->ds);
	    $res = PEAR::raiseError(ldap_error($this->ds));
	    $this->_disconnect();
	    return false;
        }

        $this->_disconnect();
        return true;

    }

    /**
     * Retrieves status of mail redirection for a user
     *
     * @param string    $user      The username of the user to check.
     * @param string    $realm     The realm of the user to check.
     *
     * @return boolean  Returns true if forwarding is enabled for the user
     *                            or false if forwarding is currently disabled.
     */

    function isEnabled($user, $realm, $password)
    {
        /* _connect() will die with Horde::fatal() upon failure. */
        $this->_connect($user, $realm, $password);

        // get current details
        $current_details = $this->_get_vacation();
        if ($current_details) {
	    return true;
        }
	return false;

        $this->_disconnect();
        return false;
    }

    /**
     * Retrieves current target of mail redirection
     *
     * @param string    $user      The username of the user.
     * @param string    $realm    The realm of the user.
     *
     * @return string   A string of current forwarding mail address or false.
     */

    function currentMessage($user, $realm, $password)
    {
        /* _connect() will die with Horde::fatal() upon failure. */
        $this->_connect($user, $realm, $password);

        $msg = $this->_get_vacation();

        $this->_disconnect();
        // Prepare the message. \n->\n\n and UTF-8 encode.
        $msg = ereg_replace("\\\\n", "\r\n", $msg); 
        $msg = mb_convert_encoding($msg, "ISO-8859-1", "UTF-8");
        return $msg;
    }

}
