<?php
/**
 * $Horde: vacation/lib/Driver/qmail.php,v 1.4.2.2 2004/08/26 19:33:59 jan Exp $
 *
 * Copyright 2001-2003 Eric Rostetter <eric.rostetter@physics.utexas.edu>
 *
 * See the enclosed file LICENSE for license information (BSD). If you
 * did not receive this file, see http://www.horde.org/bsdl.php.
 *
 * Vacation_Driver_qmail:: implements the Vacation_Driver API for ftp driven
 * qmail mail servers.  This depends on Peter Samuel's qmail-vacation 
 * program.
 *
 * @author  Eric Rostetter <eric.rostetter@physics.utexas.edu>
 * @version $Revision: 1.4.2.2 $
 * @since   Vacation 2.2
 * @package Vacation
 */

class Vacation_Driver_qmail extends Vacation_Driver {

    /** The FTP stream we open via the Horde_VFS class */
    var $_vfs;

    /** The error string returned to the user if an error occurs. */
    var $err_str;

    /** Hash containing configuration data. */
    var $params;

/**
 * Constructs a new ftp qmail Vacation_Driber object.
 *
 * @param array  $params    A hash containing connection parameters.
 */
function Vacation_Driver_qmail($params = array())
{
    $this->params = $params;
}


/**
 * Check if the realm has a specific configuration.  If not, try to fall
 * back on the default configuration.  If still not a valid configuration
 * then exit with an error.
 *
 * @param string    $realm      The realm of the user, or "default" if none.
 *                              Note: passed by reference so we can change
 *                              it's value!
 *
 */

function check_config(&$realm) {

    // If no realm passed in, or no host config for the realm passed in,
    // then we fall back to the default realm

    if ( empty($realm) || empty($this->params[$realm]['host']) ) {
       $realm = "default";
    }

    // If still no host/port, then we have a misconfigured module
    if (empty($this->params[$realm]['host']) ||
        empty($this->params[$realm]['port']) ) {
            $this->err_str = _("The module is not properly configured!");
            return false;
    }
    return true;
}

/**
 * Set the vacation notice up.
 * @param string    $user       The username to enable vacation for.
 * @param string    $realm      The realm of the user.
 * @param string    $pass       The password for the user.
 * @param string    $message    The message to install.
 * @param string    $alias      The email alias to pass to vacation
 * @return boolean  Returns true on success, false on error.
 */

    function set_vacation($user, $realm, $pass, $message, $alias) {

        // Make sure the configuration file is correct
        if (!$this->check_config($realm)) {
            return false;
        }

        // We need to find out what type of database file to use
        $conf = &$GLOBALS['conf'];
        $dbfile = VACATION_BASE . "/files/empty." .
                  $this->params['dbtype'] . ".bin";

        // Build the ftp array to pass to VFS
        $_args = array( 'hostspec' => $this->params[$realm]['host'],
                        'port' => $this->params[$realm]['port'],
                        'username' => $user,
                        'password' => $pass );

        // Create the VFS ftp driver
        require_once HORDE_BASE . '/lib/VFS.php';
        $_vfs = &Horde_VFS::singleton('ftp', $_args);
        $_vfs->setParams($_args);

        // Try to login with the username/password, no realm
        $status = $_vfs->checkCredentials();
        if (PEAR::isError($status)) {
            $this->err_str = $status->getMessage();
            $this->err_str .= '  ' .  _("Check your username and password!");
            return false;
        }

        // Set up the vacation specific files first
        $status = $_vfs->writeData('', '.vacation.msg', $message);
        if (PEAR::isError($status)) {
            $this->err_str = $status->getMessage();
            return false;
        }
        $status = $_vfs->write('', '.vacation.pag', $dbfile);
        if (PEAR::isError($status)) {
            $this->err_str = $status->getMessage();
            return false;
        }
        $status = $_vfs->write('', '.vacation.dir', $dbfile);
        if (PEAR::isError($status)) {
            $this->err_str = $status->getMessage();
            return false;
        }
        $status = $_vfs->write('', '.vacation.db', $dbfile);
        if (PEAR::isError($status)) {
            $this->err_str = $status->getMessage();
            return false;
        }

        // Parse the email address and alias address passed in.
        $my_email = $this->_make_email_address($user, $realm);
        if ($my_email === false) {
            return false;
        }
        if (!empty($alias)) {
            $alias = $this->_make_email_address($alias, $realm);
            if ($alias === false) {
                return false;
            }
        }

        // Now set up the .forward file
        if (!empty($alias) && ($alias != $my_email)) {
           $contents = "| " . $conf['vacation']['path'] .
                       " -a $alias $my_email\n./Maildir/";
        } else {
           $contents = "| " . $conf['vacation']['path'] .
                       " $my_email\n./Maildir/";
        }
        $status = $_vfs->writeData('', '.qmail', $contents);
        if (PEAR::isError($status)) {
            $this->err_str = $status->getMessage();
            return false;
        }

        // Try to change the permissions, but ignore any errors
        $_vfs->changePermissions('', '.qmail', '0600');

        $_vfs->_disconnect();
        return true;
    }

/**
 * Remove any existing vacation notices.
 *
 * @param string    $user       The user to disable vacation notices for.
 * @param string    $realm      The realm of the user.
 * @param string    $pass       The password of the user.
 *
 * @return boolean  Returns true on success, false on error.
 */

    function unset_vacation($user, $realm, $pass) {
        if (!$this->check_config($realm)) {
	   return false;
	}

        $_args = array( 'hostspec' => $this->params[$realm]['host'],
                        'port' => $this->params[$realm]['port'],
                        'username' => $user,
                        'password' => $pass );

        $_vfs = &Horde_VFS::singleton('ftp', $_args);
        $_vfs->setParams($_args);

        // Try to login with the username/password, no realm
        $status = $_vfs->checkCredentials();
        if (PEAR::isError($status)) {
            $this->err_str = $status->getMessage();
            $this->err_str .= '  ' .  _("Check your username and password!");
            return false;
        }

        $status = $_vfs->deleteFile('', '.qmail');
        if (PEAR::isError($status)) {
            $this->err_str = $status->getMessage();
            $this->err_str .= '  ' .
                  _("Maybe you didn't have a vacation notice installed?");
            return false;
        }
        $status = $_vfs->deleteFile('', '.vacation.msg');
        if (PEAR::isError($status)) {
            $this->err_str = $status->getMessage();
            return false;
        }
        // we could, but don't, check for errors on these...
        // they are more-or-less harmless without the above two files...
        $_vfs->deleteFile('', '.vacation.pag');
        $_vfs->deleteFile('', '.vacation.dir');
        $_vfs->deleteFile('', '.vacation.db');

        $_vfs->_disconnect();
        return true;
    }
}

?>
